use rumqttc::{MqttOptions, Client, QoS};
use std::time::Duration;
use std::thread;

// TODO:
//   1) Represent state of division/fieldset/field/match(maybe save/load)
//   2) Create functions to update the states and publish to MQTT based on incoming 'notifications'
//   2) Create functions to update the state that map to NOTICE_IDs and take in the notice struct
// Notifications parsed last year:
//  - NOTICE_REALTIME_SCORE_CHANGED
//  - NOTICE_FIELD_TIMER_STARTED
//  - NOTICE_FIELD_TIMER_STOPPED
//  - NOTICE_FIELD_RESET_TIMER
//  - NOTICE_FIELD_MATCH_ASSIGNED
//  - NOTICE_ACTIVE_FIELD_CHANGED
//  - NOTICE_RANKINGS_UPDATED
//  - NOTICE_EVENT_STATUS_UPDATED
//  - NOTICE_ELIM_ALLIANCE_UPDATED
//  - NOTICE_ELIM_UNAVAIL_TEAMS_UPDATED
//  - NOTICE_MATCH_LIST_UPDATED
//
//  Notice protobuf:
//  message Notice {
//    optional .NoticeId id = 1;
//    optional .MatchTuple affectedMatch = 2;
//    optional .ElimAlliance elimAlliance = 3;
//    optional .SideChallengeScore sideChallengeScore = 4;
//    optional .EventConfig eventConfig = 5;
//    optional .FieldHwState fieldState = 6;
//    optional .FieldTime fieldTime = 7;
//    optional .Field field = 8;
//    optional .MatchTimeSchedList matchTimeSchedList = 9;
//    optional .MatchTimeSchedList matchTimeSched = 10;
//    optional .FieldSetList fieldSetList = 11;
//    optional .PitDisplayList pitDisplayList = 12;
//    optional .DisplayState displayState = 13;
//    optional .AssignedObjectType.AssignedObjectEnum assignedType = 15;
//    optional .PublishOptions publishOptions = 16;
//    optional .Division division = 17;
//    optional .MatchScore matchScore = 18;
//    optional .Rankings rankings = 19;
//    optional .MatchRound affectedRound = 20;
//    optional .Award award = 21;
//    optional .FieldSet fieldSet = 22;
//    optional .TeamInfo teamInfo = 23;
//    optional double obsolete_serverTime = 24;
//    optional .TeamInspection teamInspection = 25;
//    optional int32 obsolete_serverTimeZoneInSecs = 27;
//    optional .ElimQueueList elimQueueList = 28;
//    optional .TextMessageNotice textMessage = 29;
//    optional string originatingClient = 30;
//    optional bool isLocal = 31;
//    optional .DisplaySlide displaySlide = 32;
//    optional .AssignedMatchList assignedMatchList = 33;
//    optional .MobileDeviceMatchLockList mobileMatchLocks = 34;
//    optional .ElimWinsToAdvanceList elimWinsToAdvanceList = 35;
//    optional string temporaryCode = 36;
//    optional .MatchScoreList matchScoreLog = 37;
//    optional .PublishStatus publishStatus = 38;
//    optional .MatchInfo matchInfo = 39;
//    optional .AwardList awardList = 40;
//    optional string eventName = 41;
//    optional .FieldControlStatus fieldControlStatus = 42;
//  }

use prost::Message;
use std::io::Cursor;

pub mod tm_proto {
    include!(concat!(env!("OUT_DIR"), "/tm.rs"));
}

pub fn deserialize_notice(buf: &[u8]) -> Result<tm_proto::Notice, prost::DecodeError> {
    tm_proto::Notice::decode(&mut Cursor::new(buf))
}

fn main() {
    // Set client options
    let mut mqttoptions = MqttOptions::new("rumqttc", "localhost", 1883);
    mqttoptions.set_keep_alive(Duration::from_secs(5));

    // Create a new client(10 is bounded channel capacity?)
    let (mut client, mut connection) = Client::new(mqttoptions, 10);
    client.subscribe("bridge", QoS::AtMostOnce).unwrap();

    // Create a thread to own connection and handle it's incoming messages
    let mqtt_thread = thread::spawn(move ||
        for (_, message) in connection.iter().enumerate() {
            println!("Message = {:?}", message);
        }
    );

    // Publish using the client
    for _ in 0..10 {
        client.publish("bridge/status", QoS::AtLeastOnce, false, "{\"status\": true}").unwrap();
        thread::sleep(Duration::from_millis(100));
    }

    mqtt_thread.join().expect("Failed to join mqtt thread");
}
